/* ZHDB2FILE V1.1 */
/* Earle F. Philhower, III */
/* To be used my_stdio file applications */

#include <stdio.h>
#include <stdlib.h>
#include "zlib.h"


typedef unsigned char byte;
typedef unsigned long DWORD;
typedef unsigned short WORD;

#define BYTE		unsigned char
#define USHORT		unsigned short
#define UWORD		unsigned int

#define DatabaseID	unsigned int
#define RecordID	unsigned int
#define AppID		unsigned int

#define SIZE_OF_HDB_HEADER_STRUCT		92
#define SIZE_OF_HDB_REC_HEADER_STRUCT	16

typedef struct HDB_HEADER_STRUCT 
{
    BYTE  name[32];			//31 chars + NULL
	BYTE  owner[32];		//31 chars + NULL
	USHORT version;			//used to distinguish db version on device
	USHORT dbtype;			//RFU
	UWORD attributes;		//RFU
	UWORD creation_date;	//updated after db is added to device
	UWORD backup_date;		//updated after db is backed up from device
	DatabaseID id;			//updated after db is added to device
	AppID appid;			//updated after db is added to device
	UWORD record_count;		//total records
} HDBHeader;

typedef struct HDB_REC_HEADER_STRUCT
{
	UWORD offset;			//offset to record from beginning of file
	RecordID id;			//user defined must be 0x80000000 - 0x8fffffff, else updated after db is added to device
	UWORD modify_date;		//updated after db is modified from device
	USHORT total_field;		//total fields
	BYTE category;			//record category
	BYTE attribute;			//record attributes
} HDBRecordHeader;



/* Decode the FieldSize, field_ptr will move to first char in the field*/
#define DeFieldSize(field_ptr, size, byte_use) {size = 0;               \
						byte_use = 0;           \
				    do {                                \
				      (size) |=  ((((UWORD)(*field_ptr)) >> 1) & 0x0000007F) << ((byte_use++) * 7);\
				      } while (!((*field_ptr++) & 0x01));}


void ExpandZHDB(char *filename)
{
	FILE *in;
	FILE *out;
	unsigned int i;

	HDBHeader hdb_header_buf;
	HDBRecordHeader *hdb_recheader_buf;
	int insize, outsize, byte_use;
	unsigned char inbuff[4096], outbuff[4096], *inptr;


	printf("Reading:      %s\n", filename);
	in = fopen(filename, "rb");
	if (!in) { printf("Unable to open file for reading\n"); return; }
	fread(&hdb_header_buf, sizeof(char), SIZE_OF_HDB_HEADER_STRUCT, in);
	printf("Writing:      %s (%s)\n", hdb_header_buf.name, hdb_header_buf.owner);
	out = fopen(hdb_header_buf.name, "wb");
	if (!out) { printf("Unable to open file for writing\n"); return; }
	hdb_recheader_buf = (HDBRecordHeader*)malloc(hdb_header_buf.record_count * SIZE_OF_HDB_REC_HEADER_STRUCT);
	fread(hdb_recheader_buf, sizeof(char), hdb_header_buf.record_count * SIZE_OF_HDB_REC_HEADER_STRUCT, in);
	// Start at 1 to skip header record
	for (i=1; i<hdb_header_buf.record_count; i++) {
		fseek(in, hdb_recheader_buf[i].offset, SEEK_SET);
		fread(inbuff, 1, 4096, in);
		inptr = inbuff;
		DeFieldSize(inptr, insize, byte_use);
		outsize = 4096;
		uncompress(outbuff, &outsize, inptr, insize);
		fwrite(outbuff, 1, outsize, out);
	}
	fclose(in);
	fclose(out);
}


void err()
{
	printf("ZHDB2FILE v1.0:     Earle F. Philhower, III - earle@ziplabel.com\n");
	printf("Converts ZHDB files back into their source files for use on PCs.\n\n");
	printf("usage: zhdb2file filename.hdb\n");
	exit(-1);
}

int main(int argc, char **argv)
{
	if (argc!=2) err();
	ExpandZHDB(argv[1]);
	return 0;
}

